<?php

namespace ShineSoftware\Setefi\Controller;

use Magento\Sales\Model\Order\Email\Sender\OrderSender;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\App\Config\ScopeConfigInterface;

abstract class Main extends \Magento\Framework\App\Action\Action {

    protected $_orderFactory;
    protected $_logger;
    protected $_transactionBuilder;
    protected $_checkoutHelper;
    protected $_comment;
    protected $_checkoutSession;
    protected $_pageFactory;
    protected $invoiceService;
    protected $transactionFactory;
    protected $orderSender;
    protected $orderRepository;
    protected $searchCriteriaBuilder;
    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $config;

    public function __construct(
    \Magento\Framework\App\Action\Context $context, \Magento\Sales\Model\OrderFactory $orderFactory, \ShineSoftware\Setefi\Model\Logger\Shinelogger $logger, \Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface $builder, \ShineSoftware\Setefi\Helper\Checkout $checkout, \Magento\Sales\Api\Data\OrderStatusHistoryInterface $comment, \Magento\Checkout\Model\Session $checkoutSession, \Magento\Framework\View\Result\PageFactory $resultPageFactory, \Magento\Sales\Model\Service\InvoiceService $invoiceService, \Magento\Framework\DB\TransactionFactory $transactionFactor, OrderSender $orderSender, OrderRepositoryInterface $orderRepository, SearchCriteriaBuilder $searchCriteriaBuilder, ScopeConfigInterface $config
    ) {
        $this->_orderFactory = $orderFactory;
        $this->_checkoutHelper = $checkout;
        $this->_logger = $logger;
        $this->_transactionBuilder = $builder;
        $this->_comment = $comment;
        $this->_checkoutSession = $checkoutSession;
        $this->_pageFactory = $resultPageFactory;
        $this->invoiceService = $invoiceService;
        $this->transactionFactory = $transactionFactor;
        $this->orderSender = $orderSender;
        $this->orderRepository = $orderRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->config = $config;
        parent::__construct($context);
    }

    protected function _cancelPayment($orderId = NULL, $errorMsg = '') {
        $this->_logger->debug(__METHOD__ . " - Bank Response: " . $errorMsg);
        $errorMsg = trim(strip_tags($errorMsg));
        $this->_checkoutHelper->cancelCurrentOrder($errorMsg);
        if ($this->_checkoutSession->restoreQuote()) {
            $this->_logger->debug(__METHOD__ . " - Quote has been restored! The user will be redirect to the cart!");
            return true;
        } else {
            $this->_logger->debug(__METHOD__ . " - Quote has been NOT restored! The user will be redirected to the failure page!");
            return false;
        }
    }

    protected function _createTransaction($order = null, $paymentData = array(), $payment_method = 'setefi') {
        try {
            $payment = $order->getPayment();
            $payment->setLastTransId($paymentData['paymentid']);
            $payment->setTransactionId($paymentData['paymentid']);
            $payment->setAdditionalInformation((array) $paymentData);
            $formatedPrice = $order->getBaseCurrency()->formatTxt(
                    $order->getGrandTotal()
            );
            $message = __('The authorized amount is %1.', $formatedPrice);
            $trans = $this->_transactionBuilder;
            if (isset($paymentData['result']) && $paymentData['result'] == 'CAPTURED') {
                $type = \Magento\Sales\Api\Data\TransactionInterface::TYPE_ORDER;
                $isClosed = true;
            } else {
                $type = \Magento\Sales\Api\Data\TransactionInterface::TYPE_AUTH;
                $isClosed = false;
            }
            $transaction = $trans->setPayment($payment)
                    ->setOrder($order)
                    ->setTransactionId($paymentData['paymentid'])
                    ->setAdditionalInformation(
                            [\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array) $paymentData]
                    )
                    ->setFailSafe(true)
                    ->build($type);
            $payment->addTransactionCommentsToOrder(
                    $transaction, $message
            );
            $order->setState("processing")->setStatus($this->config->getValue('payment/' . $payment_method . '/paid_order_status', 'store', $order->getStoreId()));
            $payment->setParentTransactionId(null);
            $payment->save();
            $order->setTotalPaid($order->getGrandTotal());
            $order->save();
            return $transaction->save()->getTransactionId();
        } catch (\Exception $e) {
            
        }
    }
    
    protected function _createInvoice($order) {
        if ($order->canInvoice()) {
            try {
                $this->_logger->debug(__METHOD__ . " - Trying to create invoice");
                $invoice = $this->invoiceService->prepareInvoice($order);
                $invoice->setRequestedCaptureCase(\Magento\Sales\Model\Order\Invoice::CAPTURE_OFFLINE);
                $invoice->register();
                $transactionSave = $this->transactionFactory->create()
                        ->addObject($invoice)
                        ->addObject($invoice->getOrder());
                $transactionSave->save();
            } catch (\Exception $ex) {
                $this->_logger->debug(__METHOD__ . " - Error creating invoice: " . $ex->getMessage());
            }
            
        } else {
            $this->_logger->debug(__METHOD__ . " - Cannot create invoice");
        }
    }

    protected function _getApiQuoteId() {
        $quoteId = $this->_checkoutSession->getData('apiQuoteId');
        return $quoteId;
    }

    protected function _getApiOrderId() {
        $orderId = $this->_checkoutSession->getData('apiOrderId');
        return $orderId;
    }
    
    protected function _sendOrderEmail($order) {
        if ($order->getCanSendNewEmailFlag()) {
            try {
                $this->orderSender->send($order);
            } catch (\Exception $e) {
                $this->_logger->critical($e);
            }
        }
    }
    
    protected function getOrder($orderId) {
        $searchCriteria = $this->searchCriteriaBuilder->addFilter('increment_id', $orderId, 'eq')->create();
        /** @var $orderList Magento\Sales\Model\ResourceModel\Order\Collection  */
        $orderList = $this->orderRepository->getList($searchCriteria);
        if ($orderList->getTotalCount()) {
            return $orderList->getFirstItem();
        } else {
            return NULL;
        }
    }

}

;
