<?php

namespace ShineSoftware\Setefi\Gateway\Request;

use Magento\Payment\Gateway\ConfigInterface;
use Magento\Payment\Gateway\Data\PaymentDataObjectInterface;
use Magento\Payment\Gateway\Request\BuilderInterface;
use Magento\Sales\Api\Data\OrderPaymentInterface;
use \Magento\Framework\Encryption\EncryptorInterface;

class CaptureRequest implements BuilderInterface {

    private $config;
    protected $_encryptor;

    public function __construct(
    ConfigInterface $config, EncryptorInterface $encryptor
    ) {
        $this->config = $config;
        $this->_encryptor = $encryptor;
    }

    public function build(array $buildSubject) {
        if (!isset($buildSubject['payment']) || !$buildSubject['payment'] instanceof PaymentDataObjectInterface
        ) {
            throw new \InvalidArgumentException('Payment data object should be provided');
        }
        $paymentDO = $buildSubject['payment'];
        $order = $paymentDO->getOrder();
        $payment = $paymentDO->getPayment();
        if (!$payment instanceof OrderPaymentInterface) {
            throw new \LogicException('Order payment should be provided.');
        }
        $id = $this->config->getValue('id', $order->getStoreId());
        $encrypted_password = $this->config->getValue('password', $order->getStoreId());
        $password = $this->_encryptor->decrypt($encrypted_password);
        if ($this->config->getValue('debug', $order->getStoreId()) == 1) {
            $url = $this->config->getValue('urltest', $order->getStoreId());
        } else {
            $url = $this->config->getValue('urllive', $order->getStoreId());
        }
        $data = [
            'url' => $url,
            'id' => $id,
            'password' => $password,
            'operationType' => 'confirm',
            'amount' => number_format($order->getGrandTotalAmount(), 2, '.', ''),
            'currencyCode' => $this->config->getValue('currency', $order->getStoreId()),
            'merchantOrderId' => $order->getOrderIncrementId(),
            'paymentid' => $payment->getLastTransId()
        ];
        return $data;
    }

}

;
